From 07c0831b70f88e76866dcb5458680e72d36327b6 Mon Sep 17 00:00:00 2001
From: olivierdalang <olivier.dalang@gmail.com>
Date: Sat, 16 Mar 2013 20:49:21 +0100
Subject: [PATCH] Implements sanitation of entries names

When using QgsProject::writeEntry, it's really easy to make the whole
project file unreadable just by passing a scope or key that is not a valid
XML Element name...

It's a big risk, especially for plugins. So, this implements sanitation so this can't happen.
---
 src/core/qgsproject.cpp | 24 +++++++++++++++++++++++-
 1 file changed, 23 insertions(+), 1 deletion(-)

diff --git a/src/core/qgsproject.cpp b/src/core/qgsproject.cpp
index d559dec..b9f4bb2 100644
--- a/src/core/qgsproject.cpp
+++ b/src/core/qgsproject.cpp
@@ -63,7 +63,29 @@ QStringList makeKeyTokens_( QString const &scope, QString const &key )
   // be sure to include the canonical root node
   keyTokens.push_front( "properties" );
 
-  return keyTokens;
+  //sanitize keys since an unvalid xml name will make the project file unreadable !
+  QStringList sanitizedKeyTokens = QStringList();
+  for (int i = 0; i < keyTokens.size(); ++i){
+    QString sanitizedKey = keyTokens.at(i);
+
+    //replace invalid chars
+    //TODO : this is too exclusive, this should be the regexp, but I don't manage to make it work : http://www.w3.org/TR/REC-xml/#NT-NameChar
+    //QString nameCharRegexp = QString( "[^:_A-Z_a-z0-9\\xC0-\\xD6\\xC0-\\xD6\\xF8-\\x2FF\\x370-\\x37D\\x37F-\\x1FFF\\x200C-\\x200D\\x2070-\\x218F\\x2C00-\\x2FEF\\x3001-\\xD7FF\\xF900-\\xFDCF\\xFDF0-\\xFFFD\\x10000-\\xEFFFF\\-\\.0-9\\xB7\\x0300-\\x036F\\x203F-\\x2040]" );
+    QString nameCharRegexp = QString( "[^:_A-Za-z0-9]" );
+    sanitizedKey = sanitizedKey.replace( QRegExp(nameCharRegexp) , "_" );
+
+    //prepend "_" if startchar is invalid
+    //TODO : this is too exclusive, this should be the regexp, but I don't manage to make it work : http://www.w3.org/TR/REC-xml/#NT-NameStartChar
+    //QString nameStartCharRegexp = QString( "^[^:_A-Z_a-z0-9\\xC0-\\xD6\\xC0-\\xD6\\xF8-\\x2FF\\x370-\\x37D\\x37F-\\x1FFF\\x200C-\\x200D\\x2070-\\x218F\\x2C00-\\x2FEF\\x3001-\\xD7FF\\xF900-\\xFDCF\\xFDF0-\\xFFFD\\x10000-\\xEFFFF]" );
+    QString nameStartCharRegexp = QString( "^[^:_A-Za-z]" );
+    if( sanitizedKey.contains( QRegExp(nameStartCharRegexp) ) ){
+      sanitizedKey = '_' + sanitizedKey;
+    }
+
+    sanitizedKeyTokens.append(sanitizedKey);
+  }
+
+  return sanitizedKeyTokens;
 } // makeKeyTokens_
 
 
-- 
1.7.11.msysgit.0

